==> cvstrac-1_0_7.tar.gz: cvstrac/setup.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Implementation of the Setup page
*/
#include "config.h"
#include "setup.h"

/*
** WEBPAGE: /setup
*/
void setup_page(void){
  static const char *azLink[] = {
    "index", "Home", 0
  };
  char **azResult;
  int i;
  const char *zRoot, *zOldRoot;
  const char *zLog, *zOldLog;
  const char *zAsgnto, *zOldAsgnto;
  const char *zState, *zOldState;
  char *zSubsys;

  login_check_credentials();
  if( !g.okAdmin ){
    cgi_redirect("index");
    return;
  }

  zOldRoot = db_short_query("SELECT value FROM config WHERE name='cvsroot'");
  if( zOldRoot==0 ) zOldRoot = "";
  zRoot = P("r");
  if( g.okSetup && zRoot && strcmp(zOldRoot,zRoot)!=0 ){
    db_execute("UPDATE config SET value='%q' WHERE name='cvsroot';", zRoot);
    zOldRoot = zRoot;
  }
  if( g.okSetup && P("rrh") ){
    db_execute(
      "BEGIN;"
      "DELETE FROM chng WHERE not milestone;"
      "DELETE FROM filechng;"
      "DELETE FROM file;"
      "UPDATE config SET value=0 WHERE name='historysize';"
      "COMMIT;"
    );
    history_update();
  }
  zOldLog = db_short_query("SELECT value FROM config WHERE name='logfile'");
  if( zOldLog==0 ) zOldLog = "";
  zLog = P("log");
  if( g.okSetup && zLog && strcmp(zOldLog,zLog)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='logfile';"
      "INSERT INTO config(name,value) VALUES('logfile','%q');"
      "COMMIT;",
      zLog
    );
    zOldLog = zLog;
  }
  zOldAsgnto = db_short_query("SELECT value FROM config WHERE name='assignto'");
  if( zOldAsgnto==0 ) zOldAsgnto = "";
  zAsgnto = P("asgnto");
  if( zAsgnto && strcmp(zOldAsgnto,zAsgnto)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='assignto';"
      "INSERT INTO config(name,value) VALUES('assignto','%q');"
      "COMMIT;",
      zAsgnto
    );
    zOldAsgnto = zAsgnto;
  }
  zOldState = db_short_query("SELECT value FROM config "
                             "WHERE name='initial_state'");
  if( zOldState==0 ) zOldState = "";
  zState = P("istate");
  if( zState && strcmp(zOldState,zState)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='initial_state';"
      "INSERT INTO config(name,value) VALUES('initial_state','%q');"
      "COMMIT;",
      zState
    );
    zOldState = zState;
  }
  if( P("ss") ){
    zSubsys = mprintf("%s",P("ss"));
    db_execute("BEGIN; DELETE FROM subsyst;");
    while( zSubsys[0] ){
      while( isspace(zSubsys[0]) ){ zSubsys++; }
      if( zSubsys[0]==0 ) break;
      for(i=0; zSubsys[i] && !isspace(zSubsys[i]); i++){}
      if( zSubsys[i] ){
        zSubsys[i++] = 0;
      }
      db_execute("INSERT INTO subsyst(name) VALUES('%q')", zSubsys);
      zSubsys += i;
    }
    db_execute("COMMIT;");
  }
  common_header("Configuration", azLink);
  if( g.okSetup ){
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the full pathname of the root directory of the CVS repository:
    @ <br><input type="text" name="r" size="40" value="%s(zOldRoot)">
    @ <input type="submit" value="Submit"><br>
    @ After changing the CVS repository above, you will generally want to
    @ press the following button to cause the CVSROOT/history file to be
    @ reread from the new repository.
    @ <b>WARNING:</b> Pressing the button below will erase all file change
    @ history and reconstruct it from the CVSROOT/history file of the new
    @ CVS repository.  Any edits you may have made to existing file change
    @ comments will be lost.<br>
    @ <input type="submit" name="rrh" value="Reread CVSROOT/history">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the name of a HTTP Logfile. Leave the name blank to disable logging:
    @ <br><input type="text" name="log" size="40" value="%s(zOldLog)">
    @ <input type="submit" value="Submit">
    @ </form>
    @ </p>
    @ <hr>
  }
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select a user to whom new tickets will be assigned by default:<br>
  @ Assigned To:
  azResult = db_query("SELECT id FROM user UNION SELECT '' ORDER BY id");
  cgi_v_optionmenu(0, "asgnto", zOldAsgnto, azResult);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select the initial state that new tickets are created in:<br>
  @ Initial State:
  cgi_optionmenu(0, "istate", zOldState, "New", "new", "Review", "review", 
      "Defer", "defer", "Active", "active", "Fixed", "fixed", 
      "Tested", "tested", "Closed", "closed", 0);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ A whitespace-separated list of subsystems is shown below. 
  @ You may edit this text and press apply to change the list
  @ of available subsystems.
  @ </p>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  azResult = db_query("SELECT name FROM subsyst ORDER BY name");
  if( azResult[0]==0 ){
    @ <textarea cols=16 rows=10 name="ss"></textarea>
  }else{
    @ <textarea cols=16 rows=10 name="ss">%h(azResult[0])
    for(i=1; azResult[i]; i++){
      @ %h(azResult[i])
    }
    @ </textarea>
  }
  @ <br><input type="submit" value="Submit">
  @ </form>
  @ </p>
  common_footer();
}

==> cvstrac-1_0_8.tar.gz: cvstrac/setup.c <==
/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** Implementation of the Setup page
*/
#include "config.h"
#include "setup.h"

/*
** WEBPAGE: /setup
*/
void setup_page(void){
  static const char *azLink[] = {
    "index", "Home", 0
  };
  char **azResult;
  int i;
  const char *zRoot, *zOldRoot;
  const char *zLog, *zOldLog;
  const char *zAsgnto, *zOldAsgnto;
  const char *zState, *zOldState;
  char *zSubsys;
  char *zHeader, *zFooter;

  login_check_credentials();
  if( !g.okAdmin ){
    cgi_redirect("index");
    return;
  }

  if( g.okSetup && P("header") && P("footer") ){
    db_execute(
      "REPLACE INTO config VALUES('header','%q');"
      "REPLACE INTO config VALUES('footer','%q');",
       trim_string(P("header")),
       trim_string(P("footer"))
    );
  }
  zOldRoot = db_short_query("SELECT value FROM config WHERE name='cvsroot'");
  if( zOldRoot==0 ) zOldRoot = "";
  zRoot = P("r");
  if( g.okSetup && zRoot && strcmp(zOldRoot,zRoot)!=0 ){
    db_execute("UPDATE config SET value='%q' WHERE name='cvsroot';", zRoot);
    zOldRoot = zRoot;
  }
  if( g.okSetup && P("rrh") ){
    db_execute(
      "BEGIN;"
      "DELETE FROM chng WHERE not milestone;"
      "DELETE FROM filechng;"
      "DELETE FROM file;"
      "UPDATE config SET value=0 WHERE name='historysize';"
      "COMMIT;"
    );
    history_update();
  }
  if( g.okSetup && P("import_users") ){
    user_cvs_read();
  }
  zOldLog = db_short_query("SELECT value FROM config WHERE name='logfile'");
  if( zOldLog==0 ) zOldLog = "";
  zLog = P("log");
  if( g.okSetup && zLog && strcmp(zOldLog,zLog)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='logfile';"
      "INSERT INTO config(name,value) VALUES('logfile','%q');"
      "COMMIT;",
      zLog
    );
    zOldLog = zLog;
  }
  zOldAsgnto = db_short_query("SELECT value FROM config WHERE name='assignto'");
  if( zOldAsgnto==0 ) zOldAsgnto = "";
  zAsgnto = P("asgnto");
  if( zAsgnto && strcmp(zOldAsgnto,zAsgnto)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='assignto';"
      "INSERT INTO config(name,value) VALUES('assignto','%q');"
      "COMMIT;",
      zAsgnto
    );
    zOldAsgnto = zAsgnto;
  }
  zOldState = db_short_query("SELECT value FROM config "
                             "WHERE name='initial_state'");
  if( zOldState==0 ) zOldState = "";
  zState = P("istate");
  if( zState && strcmp(zOldState,zState)!=0 ){
    db_execute(
      "BEGIN;"
      "DELETE FROM config WHERE name='initial_state';"
      "INSERT INTO config(name,value) VALUES('initial_state','%q');"
      "COMMIT;",
      zState
    );
    zOldState = zState;
  }
  if( P("ss") ){
    zSubsys = mprintf("%s",P("ss"));
    db_execute("BEGIN; DELETE FROM subsyst;");
    while( zSubsys[0] ){
      while( isspace(zSubsys[0]) ){ zSubsys++; }
      if( zSubsys[0]==0 ) break;
      for(i=0; zSubsys[i] && !isspace(zSubsys[i]); i++){}
      if( zSubsys[i] ){
        zSubsys[i++] = 0;
      }
      db_execute("INSERT INTO subsyst(name) VALUES('%q')", zSubsys);
      zSubsys += i;
    }
    db_execute("COMMIT;");
  }
  common_header("Configuration", azLink);
  if( g.okSetup ){
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the full pathname of the root directory of the CVS repository:
    @ <br><input type="text" name="r" size="40" value="%s(zOldRoot)">
    @ <input type="submit" value="Submit"><br>
    @ After changing the CVS repository above, you will generally want to
    @ press the following button to cause the CVSROOT/history file to be
    @ reread from the new repository.
    @ <b>WARNING:</b> Pressing the button below will erase all file change
    @ history and reconstruct it from the CVSROOT/history file of the new
    @ CVS repository.  Any edits you may have made to existing file change
    @ comments will be lost.<br>
    @ <input type="submit" name="rrh" value="Reread CVSROOT/history">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Use the following button to automatically create a CVSTrac user ID
    @ for every user currently named in CVSROOT/passwd.  The new users will
    @ be given the same access permissions as user "anonymous" plus check-out
    @ permission and check-in permission if CVS allows the user to write.<br>
    @ <input type="submit" name="import_users" value="Import CVS Users">
    @ </form>
    @ </p>
    @ <hr>
    @
    @ <p>
    @ <form action="%s(g.zPath)" method="POST">
    @ Enter the name of a HTTP Logfile. Leave the name blank to disable logging:
    @ <br><input type="text" name="log" size="40" value="%s(zOldLog)">
    @ <input type="submit" value="Submit">
    @ </form>
    @ </p>
    @ <hr>
  }
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select a user to whom new tickets will be assigned by default:<br>
  @ Assigned To:
  azResult = db_query("SELECT id FROM user UNION SELECT '' ORDER BY id");
  cgi_v_optionmenu(0, "asgnto", zOldAsgnto, (const char**)azResult);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  @ Select the initial state that new tickets are created in:<br>
  @ Initial State:
  cgi_optionmenu(0, "istate", zOldState, "New", "new", "Review", "review", 
      "Defer", "defer", "Active", "active", "Fixed", "fixed", 
      "Tested", "tested", "Closed", "closed", 0);
  @ <input type="submit" value="Submit">
  @ </form>
  @ </p>
  @ <hr>
  @
  @ <p>
  @ A whitespace-separated list of subsystems is shown below. 
  @ You may edit this text and press apply to change the list
  @ of available subsystems.
  @ </p>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  azResult = db_query("SELECT name FROM subsyst ORDER BY name");
  if( azResult[0]==0 ){
    @ <textarea cols=16 rows=10 name="ss"></textarea>
  }else{
    @ <textarea cols=16 rows=10 name="ss">%h(azResult[0])
    for(i=1; azResult[i]; i++){
      @ %h(azResult[i])
    }
    @ </textarea>
  }
  @ <br><input type="submit" value="Submit">
  @ </form>
  @ <hr>
  @
  @ <p>
  @ Enter HTML used for the header and footer of every page below.
  @ (If you leave these entries blank, default headers and/or footers
  @ are used.)  The following substitutions occur with the text:
  @ </p>
  @ <blockquote>
  @ <dl compact>
  @ <dt><b>%%N</b></dt><dd>The name of the project</dd>
  @ <dt><b>%%T</b></dt><dd>The title of the current page</dd>
  @ <dt><b>%%V</b></dt><dd>The version number of CVSTrac</dd>
  @ <dt><b>%%%%</b></dt><dd>The literal character "<b>%%</b>"</dd>
  @ </dl>
  @ </blockquote>
  @
  @ <p>
  @ <form action="%s(g.zPath)" method="POST">
  zHeader = db_short_query("SELECT value FROM config WHERE name='header'");
  zFooter = db_short_query("SELECT value FROM config WHERE name='footer'");
  @ Header:<br>
  @ <textarea cols=80 rows=10 name="header">%h(zHeader)</textarea><br>
  @ Footer:<br>
  @ <textarea cols=80 rows=10 name="footer">%h(zFooter)</textarea><br>
  if( g.okSetup ){
    @ <input type="submit" value="Submit">
  }else{
    @ (Only the "setup" user can change the header and footer text)
  }
  @ </form>
  @ </p>
  common_footer();
}
